@@ -1,6 +1,6 @@
 /* ----------------------------------------------------------------------- *
  *   
- *   Copyright 1996-2009 The NASM Authors - All Rights Reserved
+ *   Copyright 1996-2016 The NASM Authors - All Rights Reserved
  *   See the file AUTHORS included with the NASM distribution for
  *   the specific copyright holders.
  *
@@ -570,11 +570,15 @@ static struct ExtBack {
 
 static struct Segment {
     struct Segment *next;
+    char *name;
     int32_t index;                 /* the NASM segment id */
     int32_t obj_index;             /* the OBJ-file segment index */
     struct Group *grp;          /* the group it beint32_ts to */
     uint32_t currentpos;
     int32_t align;                 /* can be SEG_ABS + absolute addr */
+    struct Public *pubhead, **pubtail, *lochead, **loctail;
+    char *segclass, *overlay;   /* `class' is a C++ keyword :-) */
+    ObjRecord *orp;
     enum {
         CMB_PRIVATE = 0,
         CMB_PUBLIC = 2,
@@ -582,10 +586,6 @@ static struct Segment {
         CMB_COMMON = 6
     } combine;
     bool use32;                 /* is this segment 32-bit? */
-    struct Public *pubhead, **pubtail, *lochead, **loctail;
-    char *name;
-    char *segclass, *overlay;   /* `class' is a C++ keyword :-) */
-    ObjRecord *orp;
 } *seghead, **segtail, *obj_seg_needs_update;
 
 static struct Group {
@@ -625,12 +625,13 @@ static struct ExpDef {
 static int32_t obj_entry_seg, obj_entry_ofs;
 
 struct ofmt of_obj;
+static struct dfmt borland_debug_form;
 
 /* The current segment */
 static struct Segment *current_seg;
 
 static int32_t obj_segment(char *, int, int *);
-static void obj_write_file(int debuginfo);
+static void obj_write_file(void);
 static int obj_directive(enum directives, char *, int);
 
 static void obj_init(void)
@@ -667,10 +668,11 @@ static int obj_set_info(enum geninfo type, char **val)
 
     return 0;
 }
-static void obj_cleanup(int debuginfo)
+
+static void obj_cleanup(void)
 {
-    obj_write_file(debuginfo);
-    of_obj.current_dfmt->cleanup();
+    obj_write_file();
+    dfmt->cleanup();
     while (seghead) {
         struct Segment *segtmp = seghead;
         seghead = seghead->next;
@@ -840,7 +842,7 @@ static void obj_deflabel(char *name, int32_t segment,
     if (!any_segs && segment == first_seg) {
         int tempint;            /* ignored */
         if (segment != obj_segment("__NASMDEFSEG", 2, &tempint))
-            nasm_error(ERR_PANIC, "strange segment conditions in OBJ driver");
+            nasm_panic(0, "strange segment conditions in OBJ driver");
     }
 
     for (seg = seghead; seg && is_global; seg = seg->next)
@@ -969,7 +971,7 @@ static void obj_deflabel(char *name, int32_t segment,
                 stdscan_reset();
                 stdscan_set(special);
                 tokval.t_type = TOKEN_INVALID;
-                e = evaluate(stdscan, NULL, &tokval, NULL, 1, nasm_error, NULL);
+                e = evaluate(stdscan, NULL, &tokval, NULL, 1, NULL);
                 if (e) {
                     if (!is_simple(e))
                         nasm_error(ERR_NONFATAL, "cannot use relocatable"
@@ -1046,7 +1048,7 @@ static void obj_out(int32_t segto, const void *data,
     if (!any_segs) {
         int tempint;            /* ignored */
         if (segto != obj_segment("__NASMDEFSEG", 2, &tempint))
-            nasm_error(ERR_PANIC, "strange segment conditions in OBJ driver");
+            nasm_panic(0, "strange segment conditions in OBJ driver");
     }
 
     /*
@@ -1056,7 +1058,7 @@ static void obj_out(int32_t segto, const void *data,
         if (seg->index == segto)
             break;
     if (!seg)
-        nasm_error(ERR_PANIC, "code directed to nonexistent segment?");
+        nasm_panic(0, "code directed to nonexistent segment?");
 
     orp = seg->orp;
     orp->parm[0] = seg->currentpos;
@@ -1086,6 +1088,9 @@ static void obj_out(int32_t segto, const void *data,
     {
         int rsize;
 
+        if (type == OUT_ADDRESS)
+            size = abs((int)size);
+
         if (segment == NO_SEG && type != OUT_ADDRESS)
             nasm_error(ERR_NONFATAL, "relative call to absolute address not"
                   " supported by OBJ format");
@@ -1190,7 +1195,7 @@ static void obj_write_fixup(ObjRecord * orp, int bytes,
         locat = FIX_16_SELECTOR;
         seg--;
         if (bytes != 2)
-            nasm_error(ERR_PANIC, "OBJ: 4-byte segment base fixup got"
+            nasm_panic(0, "OBJ: 4-byte segment base fixup got"
                   " through sanity check");
     } else {
         base = false;
@@ -1236,7 +1241,7 @@ static void obj_write_fixup(ObjRecord * orp, int bytes,
             if (eb)
                 method = 6, e = eb->exts[i], tidx = e->index;
             else
-                nasm_error(ERR_PANIC,
+                nasm_panic(0,
                       "unrecognised segment value in obj_write_fixup");
         }
     }
@@ -1295,7 +1300,7 @@ static void obj_write_fixup(ObjRecord * orp, int bytes,
                 if (eb)
                     method |= 0x20, fidx = eb->exts[i]->index;
                 else
-                    nasm_error(ERR_PANIC,
+                    nasm_panic(0,
                           "unrecognised WRT value in obj_write_fixup");
             }
         }
@@ -1444,7 +1449,7 @@ static int32_t obj_segment(char *name, int pass, int *bits)
                         if (!strcmp(grp->name, "FLAT"))
                             break;
                     if (!grp)
-                        nasm_error(ERR_PANIC, "failure to define FLAT?!");
+                        nasm_panic(0, "failure to define FLAT?!");
                 }
                 seg->grp = grp;
             } else if (!nasm_strnicmp(p, "class=", 6))
@@ -1913,7 +1918,7 @@ static void obj_filename(char *inname, char *outname)
     standard_extension(inname, outname, ".obj");
 }
 
-static void obj_write_file(int debuginfo)
+static void obj_write_file(void)
 {
     struct Segment *seg, *entry_seg_ptr = 0;
     struct FileName *fn;
@@ -1925,6 +1930,7 @@ static void obj_write_file(int debuginfo)
     struct ExpDef *export;
     int lname_idx;
     ObjRecord *orp;
+    const bool debuginfo = (dfmt == &borland_debug_form);
 
     /*
      * Write the THEADR module header.
@@ -2393,7 +2399,7 @@ static void obj_fwrite(ObjRecord * orp)
     len = orp->committed + 1;
     cksum += (len & 0xFF) + ((len >> 8) & 0xFF);
     fwriteint16_t(len, ofile);
-    fwrite(orp->buf, 1, len - 1, ofile);
+    nasm_write(orp->buf, len-1, ofile);
     for (ptr = orp->buf; --len; ptr++)
         cksum += *ptr;
     fputc((-cksum) & 0xFF, ofile);
@@ -2401,7 +2407,7 @@ static void obj_fwrite(ObjRecord * orp)
 
 extern macros_t obj_stdmac[];
 
-void dbgbi_init(void)
+static void dbgbi_init(void)
 {
     fnhead = NULL;
     fntail = &fnhead;
@@ -2454,7 +2460,7 @@ static void dbgbi_linnum(const char *lnfname, int32_t lineno, int32_t segto)
     if (!any_segs) {
         int tempint;            /* ignored */
         if (segto != obj_segment("__NASMDEFSEG", 2, &tempint))
-            nasm_error(ERR_PANIC, "strange segment conditions in OBJ driver");
+            nasm_panic(0, "strange segment conditions in OBJ driver");
     }
 
     /*
@@ -2464,7 +2470,7 @@ static void dbgbi_linnum(const char *lnfname, int32_t lineno, int32_t segto)
         if (seg->index == segto)
             break;
     if (!seg)
-        nasm_error(ERR_PANIC, "lineno directed to nonexistent segment?");
+        nasm_panic(0, "lineno directed to nonexistent segment?");
 
 /*    for (fn = fnhead; fn; fn = fnhead->next) */
     for (fn = fnhead; fn; fn = fn->next)        /* fbk - Austin Lunnen - John Fine */
@@ -2496,20 +2502,16 @@ static void dbgbi_deflabel(char *name, int32_t segment,
 
     (void)special;
 
+    /*
+     * Note: ..[^@] special symbols are filtered in labels.c
+     */
+
     /*
      * If it's a special-retry from pass two, discard it.
      */
     if (is_global == 3)
         return;
 
-    /*
-     * First check for the double-period, signifying something
-     * unusual.
-     */
-    if (name[0] == '.' && name[1] == '.' && name[2] != '@') {
-        return;
-    }
-
     /*
      * Case (i):
      */
@@ -2624,6 +2626,7 @@ struct ofmt of_obj = {
     "MS-DOS 16-bit/32-bit OMF object files",
     "obj",
     0,
+    32,
     borland_debug_arr,
     &borland_debug_form,
     obj_stdmac,
